/*
  Copyright 2009 CNRS-LAAS

  Author: Oussama Kanoun
*/

#ifndef HPP_LOS_GEOMETRY_H
#define HPP_LOS_GEOMETRY_H

#include "MatrixAbstractLayer/MatrixAbstractLayer.h"

class LosGeometry
{
    public:
        virtual void transform() = 0;
        virtual const matrix4d* associatedTransformation() const;
        virtual ~LosGeometry(){};
    protected:
        const matrix4d* attFrame;
};

class LosHalfspace: virtual public LosGeometry
{
    public:
        LosHalfspace ( const vector3d& inPoint, const vector3d& inOuterNormalN,const matrix4d* refFrame );
        void transform();
        const vector3d& borderPoint() const;
        const vector3d& outerNormal() const;
    private:
        vector3d attPoint, attAbsPoint;
        vector3d attNormal, attAbsNormal;
        matrix4d m;
};

class LosCylinder: virtual public LosGeometry
{
    public:
        LosCylinder ( const vector3d& inPointA, const vector3d& inPointB, double inRadius, const matrix4d* refFrame = 0 );
        double length() const ;
        double radius() const ;
        const vector3d& localA() const ;
        const vector3d& localB() const;
        const vector3d& localAB() const ;
        const vector3d& absoluteA() const;
        const vector3d& absoluteB() const ;
        const vector3d& absoluteAB() const ;
        void transform();
    protected:
        vector3d locA,locB,locAB;
        vector3d A,B,AB;
        double attLength;
        double attRadius;
        matrix4d m;
};

#endif
