/*
  Copyright 2009 CNRS-LAAS

  Author: Oussama Kanoun
*/
#ifndef HPP_LOS_CONE_TASK_H
#define HPP_LOS_CONE_TASK_H

#include "hpplocalstepper/tasks/hppLosTask.hh"
class ChppLosRobotLink;

/**
A task to put a unitary vector attached to a body within a limit vector field defined by a minimum cosine value with respect to a unitary vector given in world frame
\ingroup tasks
 */
class ChppLosConeTask: public ChppLosTaskOnJoint
{
    public:

        /**
        \brief Constructor
         */
        ChppLosConeTask ( ChppLosCentipede& inCentipede, ChppLosRobotLink& inRobotLink, CjrlJoint& inJoint, const vector3d& inLocalVector, const vector3d& inWorldVector, double maxCosine);
        
        void setMargins ( double inDi, double inDs, double inXi );
        /**
        \brief Get reference point in reference joint
         */
        const vector3d& localVector();
        /**
        \brief Get the coordinates of a point in the target plane
         */
        const vector3d& worldVector();
        /**
        \brief Set reference point in reference joint local coordinates
         */
        void localVector ( const vector3d& inVec );
        /**
        \brief Set reference point in reference joint local coordinates
         */
        void minCosine ( double inVal );
        /**
        \brief Set a point in the target plane
         */
        void worldVector ( const vector3d& inVec );
        /**
        \brief Set the maximum linear velocity of the effector point
         */
        void maxVelocityNorm ( double inMV );
        /**
        \brief Return true
         */
        bool isEqualityTask() const {return false;};
        /**
        \brief Inherited
         */
        void computeDifferentialSystem();
        /**
        \brief Inherited
         */
        const vectorN& currentValue();

    private:
        void equivAsymMat3 ( const vector3d& inVec, matrixNxP& outMat );
        bool resize();
        double attMinCosine;
        vector3d attCurrentVector;
        ChppLosRobotLink* attRobotLink;
        vector3d attLocalVector, attWorldVector;
        double attMaxVelocity;
        matrixNxP attTempJacobian, attPlaneNormalM;
        matrix3d m3;
        double attXi,attDi,attDs;
};

#endif
