/*
  Copyright 2009 CNRS-LAAS

  Author: Oussama Kanoun
*/

#ifndef HPP_LOS_CONFIG_TASK_H
#define HPP_LOS_CONFIG_TASK_H

#include "hpplocalstepper/tasks/hppLosTask.hh"
class ChppLosLink;

/**
A task to reach a given configuration for a ChppLosCentipede object. Caution: This task does not resize itself when the centipede assembly changes.
\ingroup tasks
 */
class ChppLosConfigurationTask: public ChppLosTask
{
    public:
       
        /**
        \brief Constructor
         */
        ChppLosConfigurationTask(ChppLosCentipede& inCentipede, const vectorN& inTarget, const vectorN& inMask);
        /**
        \brief Get the target configuration
         */
        const vectorN& targetConfiguration();
        /**
        \brief Set the target configuration
         */
        bool targetConfiguration(const vectorN& inV);
        /**
        \brief Get the configuration mask
         */
        const vectorN& mask();
        /**
        \brief Set the configuration mask
         */
        bool mask(const vectorN& inV);
        /**
        \brief Return true
         */
        bool isEqualityTask() const {return true;};
        /**
        \brief Inherited
         */
        void computeDifferentialSystem();
        /**
        \brief Set the maximum velocity norm for any joint
         */
        void maxVelocityNorm ( double inMV );
        /**
        \brief Inherited
         */
        const vectorN& currentValue();

    private:
        vectorN attConfig;
        double attMaxVelocity;
        vectorN attTarget,attMask;
};

#endif
