#include "boost/numeric/ublas/vector_proxy.hpp"
#include "boost/numeric/ublas/matrix_proxy.hpp"
#include "hpp/localstepper/task/com-task.hh"
#include "hpp/localstepper/centipede.hh"
#include "hpp/localstepper/link.hh"
#include "hpp/hik/linear-system.hh"
#include "hpp/gik/robot/standing-robot.hh"
#include "robotDynamics/jrlHumanoidDynamicRobot.h"

using namespace boost::numeric::ublas;

ChppLosComTask::ChppLosComTask (ChppLosCentipede& inCentipede, ChppLosRobotLink& inRobotLink, double inX, double inY):ChppLosTask(inCentipede)
{
    attRobotLink = &inRobotLink;
    attTargetX = inX;
    attTargetY = inY;

    attTempJacobian.resize(3,attCentipede->numberDof());
    attTempJacobian.clear();
    attConstraint->matrix().resize(2,attCentipede->numberDof());
    attConstraint->matrix().clear();
    attConstraint->vector().resize(2);
    attValue.resize(2);
    attMaxVelocity = 1e10;
}

bool ChppLosComTask::resize()
{
    if(ChppLosTask::resize())
    {
        attTempJacobian.resize(3,attCentipede->numberDof());
        return true;
    }
    return false;
}

void ChppLosComTask::targetXY(double inX, double inY)
{
    attTargetX = inX;
    attTargetY = inY;
}

void ChppLosComTask::maxVelocityNorm(double inMV)
{
    attMaxVelocity = (inMV>0.0)?inMV:0.0;
}

const vectorN& ChppLosComTask::currentValue()
{
    const vector3d& com = attRobotLink->standingRobot().robot()->positionCenterOfMass();
    attValue(0) = attTargetX - com[0];
    attValue(1) = attTargetY - com[1];
    return attValue;
}

void ChppLosComTask::computeDifferentialSystem()
{
    resize();
    const vector3d& com = attRobotLink->standingRobot().robot()->positionCenterOfMass();
    attConstraint->vector()(0) = attTargetX - com[0];
    attConstraint->vector()(1) = attTargetY - com[1];
    
    double nor = norm_2(attConstraint->vector());
    if (nor > attMaxVelocity)
        attConstraint->vector() = attMaxVelocity*attConstraint->vector()/nor;
    
    attTempJacobian.clear();
    attRobotLink->standingRobot().robot()->getJacobianCenterOfMass(*attRobotLink->rootJoint(), attTempJacobian, attRobotLink->rank(), false );
    ChppLosLink* plink = attRobotLink->parent();
    
    while ( plink!=0 )
    {
        plink->getPositionJacobian ( com,attTempJacobian,plink->rank() );
        plink = plink->parent();
    }
    noalias(row(attConstraint->matrix(),0)) = row(attTempJacobian,0);
    noalias(row(attConstraint->matrix(),1)) = row(attTempJacobian,1);
}

