#ifndef HRP2_WRAPPER_H
#define HRP2_WRAPPER_H

#include <vector>
class LosRobotCylinder;
class ChppLosCentipede;
class ChppLosRobotLink;

/**
Constructs and returns cylinders to wrap the geometry of the rigid body links composing HRP2
 */
class HRP2Wrapper
{
    public:
       /**
        \brief Constructor
        */
        HRP2Wrapper(ChppLosCentipede& inCentipede, ChppLosRobotLink& inRobotLink);
       
        LosRobotCylinder& head(){return *attHead;}
        
        LosRobotCylinder& chest(){return *attChest;}
        
        LosRobotCylinder& hips(){return *attHips;}
        
        LosRobotCylinder& upperlegR(){return *attUpperlegR;}
        
        LosRobotCylinder& lowerlegR(){return *attLowerlegR;}
        
        LosRobotCylinder& upperlegL(){return *attUpperlegL;}
        
        LosRobotCylinder& lowerlegL(){return *attLowerlegL;}
        
        LosRobotCylinder& upperarmR(){return *attUpperarmR;}
        
        LosRobotCylinder& shoulderR(){return *attShoulderR;}
        
        LosRobotCylinder& shoulderL(){return *attShoulderL;}
        
        LosRobotCylinder& lowerarmR(){return *attLowerarmR;}
        
        LosRobotCylinder& upperarmL(){return *attUpperarmL;}
        
        LosRobotCylinder& lowerarmL(){return *attLowerarmL;}
        
        LosRobotCylinder& handL(){return *attHandL;}
        
        LosRobotCylinder& handR(){return *attHandR;}
        
        LosRobotCylinder& openhandL(){return *attOpenhandL;}
        
        LosRobotCylinder& openhandR(){return *attOpenhandR;}
        
        std::vector<LosRobotCylinder*> cylinders()
        {
            return attCylinders;
        }
        
        void updateModel();
        
        ~HRP2Wrapper();
        
    private:
        
        std::vector< LosRobotCylinder*> attCylinders;
        ChppLosCentipede *attCentipede;
        ChppLosRobotLink *attLink;

        LosRobotCylinder* attHead;
        LosRobotCylinder* attChest;
        LosRobotCylinder* attHips;
        LosRobotCylinder* attUpperlegR;
        LosRobotCylinder* attShoulderR;
        LosRobotCylinder* attShoulderL;
        LosRobotCylinder* attLowerlegR;
        LosRobotCylinder* attUpperlegL;
        LosRobotCylinder* attLowerlegL;
        LosRobotCylinder* attUpperarmR;
        LosRobotCylinder* attLowerarmR;
        LosRobotCylinder* attUpperarmL;
        LosRobotCylinder* attLowerarmL;
        LosRobotCylinder* attHandL;
        LosRobotCylinder* attHandR;
        LosRobotCylinder* attOpenhandL;
        LosRobotCylinder* attOpenhandR;
};

#endif
